@@ -21,6 +21,7 @@ from django.contrib.contenttypes.models import ContentType
 from django.db.models.signals import class_prepared, post_save, pre_save
 from django.utils.translation import ugettext_lazy as _
 
+from feincms import extensions
 from feincms.contrib.fields import JSONField
 from feincms.models import ContentProxy
 
@@ -82,7 +83,7 @@ class TrackerContentProxy(ContentProxy):
                 dct = ContentType.objects.get_for_model(fct)
 
                 # Rely on non-negative primary keys
-                map[-dct.id] = idx # From-inventory map
+                map[-dct.id] = idx  # From-inventory map
                 map[idx] = dct.id  # To-inventory map
 
             _translation_map_cache[cls] = map
@@ -98,7 +99,7 @@ class TrackerContentProxy(ContentProxy):
 
         return dict((region, [
             (pk, map[-ct]) for pk, ct in items
-            ]) for region, items in inventory.items() if region!='_version_')
+            ]) for region, items in inventory.items() if region != '_version_')
 
     def _to_inventory(self, counts):
         map = self._translation_map()
@@ -109,6 +110,7 @@ class TrackerContentProxy(ContentProxy):
         inventory['_version_'] = INVENTORY_VERSION
         return inventory
 
+
 # ------------------------------------------------------------------------
 def class_prepared_handler(sender, **kwargs):
     # It might happen under rare circumstances that not all model classes
@@ -119,6 +121,7 @@ def class_prepared_handler(sender, **kwargs):
     _translation_map_cache = {}
 class_prepared.connect(class_prepared_handler)
 
+
 # ------------------------------------------------------------------------
 def tree_post_save_handler(sender, instance, **kwargs):
     """
@@ -129,19 +132,23 @@ def tree_post_save_handler(sender, instance, **kwargs):
     # inheritance has been customized.
     instance.get_descendants(include_self=True).update(_ct_inventory=None)
 
+
 # ------------------------------------------------------------------------
 def single_pre_save_handler(sender, instance, **kwargs):
     """Clobber the _ct_inventory attribute of this object"""
 
     instance._ct_inventory = None
 
+
 # ------------------------------------------------------------------------
-def register(cls, admin_cls):
-    cls.add_to_class('_ct_inventory', JSONField(_('content types'), editable=False, blank=True, null=True))
-    cls.content_proxy_class = TrackerContentProxy
+class Extension(extensions.Extension):
+    def handle_model(self):
+        self.model.add_to_class('_ct_inventory', JSONField(
+            _('content types'), editable=False, blank=True, null=True))
+        self.model.content_proxy_class = TrackerContentProxy
 
-    pre_save.connect(single_pre_save_handler, sender=cls)
-    if hasattr(cls, 'get_descendants'):
-        post_save.connect(tree_post_save_handler, sender=cls)
+        pre_save.connect(single_pre_save_handler, sender=self.model)
+        if hasattr(self.model, 'get_descendants'):
+            post_save.connect(tree_post_save_handler, sender=self.model)
 
 # ------------------------------------------------------------------------
