@@ -12,6 +12,7 @@ from django.db.models.signals import post_delete
 from django.dispatch.dispatcher import receiver
 from django.template.defaultfilters import slugify
 from django.utils import timezone
+from django.utils.encoding import python_2_unicode_compatible
 from django.utils.translation import ugettext_lazy as _
 
 from feincms import settings
@@ -21,16 +22,19 @@ from feincms.translations import (TranslatedObjectMixin, Translation,
 
 from . import logger
 
+
 # ------------------------------------------------------------------------
 class CategoryManager(models.Manager):
     """
     Simple manager which exists only to supply ``.select_related("parent")``
-    on querysets since we can't even __unicode__ efficiently without it.
+    on querysets since we can't even __str__ efficiently without it.
     """
     def get_query_set(self):
         return super(CategoryManager, self).get_query_set().select_related("parent")
 
+
 # ------------------------------------------------------------------------
+@python_2_unicode_compatible
 class Category(models.Model):
     """
     These categories are meant primarily for organizing media files in the
@@ -51,7 +55,7 @@ class Category(models.Model):
 
     objects = CategoryManager()
 
-    def __unicode__(self):
+    def __str__(self):
         if self.parent_id:
             return u'%s - %s' % (self.parent.title, self.title)
 
@@ -66,7 +70,7 @@ class Category(models.Model):
 
     def path_list(self):
         if self.parent is None:
-            return [ self ]
+            return [self]
         p = self.parent.path_list()
         p.append(self)
         return p
@@ -74,7 +78,9 @@ class Category(models.Model):
     def path(self):
         return ' - '.join((f.title for f in self.path_list()))
 
+
 # ------------------------------------------------------------------------
+@python_2_unicode_compatible
 class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
     """
     Abstract media file class. Includes the :class:`feincms.models.ExtensionsMixin`
@@ -85,7 +91,7 @@ class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
     type = models.CharField(_('file type'), max_length=12, editable=False, choices=())
     created = models.DateTimeField(_('created'), editable=False, default=timezone.now)
     copyright = models.CharField(_('copyright'), max_length=200, blank=True)
-    file_size  = models.IntegerField(_("file size"), blank=True, null=True, editable=False)
+    file_size = models.IntegerField(_("file size"), blank=True, null=True, editable=False)
 
     categories = models.ManyToManyField(Category, verbose_name=_('categories'),
                                         blank=True, null=True)
@@ -99,8 +105,8 @@ class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
 
     objects = TranslatedObjectManager()
 
-    filetypes = [ ]
-    filetypes_dict = { }
+    filetypes = []
+    filetypes_dict = {}
 
     @classmethod
     def reconfigure(cls, upload_to=None, storage=None):
@@ -117,7 +123,7 @@ class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
     @classmethod
     def register_filetypes(cls, *types):
         cls.filetypes[0:0] = types
-        choices = [ t[0:2] for t in cls.filetypes ]
+        choices = [t[0:2] for t in cls.filetypes]
         cls.filetypes_dict = dict(choices)
         cls._meta.get_field('type').choices[:] = choices
 
@@ -126,7 +132,7 @@ class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
         if self.file:
             self._original_file_name = self.file.name
 
-    def __unicode__(self):
+    def __str__(self):
         trans = None
 
         try:
@@ -137,7 +143,7 @@ class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
             pass
 
         if trans:
-            trans = unicode(trans)
+            trans = u'%s' % trans
             if trans.strip():
                 return trans
         return os.path.basename(self.file.name)
@@ -172,12 +178,12 @@ class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
         if self.file:
             try:
                 self.file_size = self.file.size
-            except (OSError, IOError, ValueError), e:
+            except (OSError, IOError, ValueError) as e:
                 logger.error("Unable to read file size for %s: %s" % (self, e))
 
         super(MediaFileBase, self).save(*args, **kwargs)
 
-        logger.info("Saved mediafile %d (%s, type %s, %d bytes)" % (self.id, self.file.name, self.type, self.file_size))
+        logger.info("Saved mediafile %d (%s, type %s, %d bytes)" % (self.id, self.file.name, self.type, self.file_size or 0))
 
         # User uploaded a new file. Try to get rid of the old file in
         # storage, to avoid having orphaned files hanging around.
@@ -193,14 +199,15 @@ class MediaFileBase(models.Model, ExtensionsMixin, TranslatedObjectMixin):
             name = self.file.name
         try:
             self.file.storage.delete(name)
-        except Exception, e:
+        except Exception as e:
             logger.warn("Cannot delete media file %s: %s" % (name, e))
 
+
 # ------------------------------------------------------------------------
 MediaFileBase.register_filetypes(
         # Should we be using imghdr.what instead of extension guessing?
         ('image', _('Image'), lambda f: re.compile(r'\.(bmp|jpe?g|jp2|jxr|gif|png|tiff?)$', re.IGNORECASE).search(f)),
-        ('video', _('Video'), lambda f: re.compile(r'\.(mov|m[14]v|mp4|avi|mpe?g|qt|ogv|wmv)$', re.IGNORECASE).search(f)),
+        ('video', _('Video'), lambda f: re.compile(r'\.(mov|m[14]v|mp4|avi|mpe?g|qt|ogv|wmv|flv)$', re.IGNORECASE).search(f)),
         ('audio', _('Audio'), lambda f: re.compile(r'\.(au|mp3|m4a|wma|oga|ram|wav)$', re.IGNORECASE).search(f)),
         ('pdf', _('PDF document'), lambda f: f.lower().endswith('.pdf')),
         ('swf', _('Flash'), lambda f: f.lower().endswith('.swf')),
@@ -210,23 +217,23 @@ MediaFileBase.register_filetypes(
         ('doc', _('Microsoft Word'), lambda f: re.compile(r'\.docx?$', re.IGNORECASE).search(f)),
         ('xls', _('Microsoft Excel'), lambda f: re.compile(r'\.xlsx?$', re.IGNORECASE).search(f)),
         ('ppt', _('Microsoft PowerPoint'), lambda f: re.compile(r'\.pptx?$', re.IGNORECASE).search(f)),
-        ('other', _('Binary'), lambda f: True), # Must be last
+        ('other', _('Binary'), lambda f: True),  # Must be last
     )
 
+
 # ------------------------------------------------------------------------
 class MediaFile(MediaFileBase):
-    @classmethod
-    def register_extension(cls, register_fn):
-        from .admin import MediaFileAdmin
+    pass
 
-        register_fn(cls, MediaFileAdmin)
 
 @receiver(post_delete, sender=MediaFile)
 def _mediafile_post_delete(sender, instance, **kwargs):
     instance.delete_mediafile()
     logger.info("Deleted mediafile %d (%s)" % (instance.id, instance.file.name))
 
+
 # ------------------------------------------------------------------------
+@python_2_unicode_compatible
 class MediaFileTranslation(Translation(MediaFile)):
     """
     Translated media file caption and description.
@@ -238,9 +245,9 @@ class MediaFileTranslation(Translation(MediaFile)):
     class Meta:
         verbose_name = _('media file translation')
         verbose_name_plural = _('media file translations')
+        unique_together = ('parent', 'language_code')
 
-    def __unicode__(self):
+    def __str__(self):
         return self.caption
 
 #-------------------------------------------------------------------------
-#-------------------------------------------------------------------------
