@@ -9,10 +9,7 @@
 
 from __future__ import absolute_import
 
-try:
-    import json
-except ImportError:
-    from django.utils import simplejson as json  # Python 2.5
+import json
 import zipfile
 import os
 import time
@@ -23,9 +20,11 @@ from django.utils import timezone
 
 from .models import Category, MediaFile, MediaFileTranslation
 
+
 # ------------------------------------------------------------------------
 export_magic = 'feincms-export-01'
 
+
 # ------------------------------------------------------------------------
 def import_zipfile(category_id, overwrite, data):
     """
@@ -103,8 +102,8 @@ def import_zipfile(category_id, overwrite, data):
                         for tr in info['translations']:
                             found_metadata = True
                             mt, mt_created = MediaFileTranslation.objects.get_or_create(parent=mf, language_code=tr['lang'])
-                            mt.caption       = tr['caption']
-                            mt.description   = tr.get('description', None)
+                            mt.caption = tr['caption']
+                            mt.description = tr.get('description', None)
                             mt.save()
 
                         # Add categories
@@ -114,7 +113,7 @@ def import_zipfile(category_id, overwrite, data):
 
                 if not found_metadata:
                     mt = MediaFileTranslation()
-                    mt.parent  = mf
+                    mt.parent = mf
                     mt.caption = fname.replace('_', ' ')
                     mt.save()
 
@@ -125,9 +124,10 @@ def import_zipfile(category_id, overwrite, data):
 
     return count
 
+
 # ------------------------------------------------------------------------
 def export_zipfile(site, queryset):
-    now  = timezone.now()
+    now = timezone.now()
     zip_name = "export_%s_%04d%02d%02d.zip" % (slugify(site.domain), now.year, now.month, now.day)
 
     zip_data = open(os.path.join(django_settings.MEDIA_ROOT, zip_name), "w")
@@ -139,23 +139,34 @@ def export_zipfile(site, queryset):
         for cat in mf.categories.all():
             used_categories.update(cat.path_list())
 
-    info = { 'export_magic': export_magic,
-             'categories': [ { 'id': cat.id, 'title': cat.title, 'slug': cat.slug, 'parent': cat.parent_id or 0, 'level': len(cat.path_list()) } for cat in used_categories ],
-            }
+    info = {
+        'export_magic': export_magic,
+        'categories': [{
+            'id': cat.id,
+            'title': cat.title,
+            'slug': cat.slug,
+            'parent': cat.parent_id or 0,
+            'level': len(cat.path_list()),
+            } for cat in used_categories],
+        }
     zip_file.comment = json.dumps(info)
 
     for mf in queryset:
         ctime = time.localtime(os.stat(mf.file.path).st_ctime)
         info = json.dumps({
             'copyright': mf.copyright,
-            'categories': [ cat.id for cat in mf.categories.all() ],
-            'translations': [
-                { 'lang': t.language_code, 'caption': t.caption, 'description': t.description }
-                    for t in mf.translations.all() ],
+            'categories': [cat.id for cat in mf.categories.all()],
+            'translations': [{
+                'lang': t.language_code,
+                'caption': t.caption,
+                'description': t.description,
+                } for t in mf.translations.all()],
             })
 
         with open(mf.file.path, "r") as file_data:
-            zip_info = zipfile.ZipInfo(filename=mf.file.name, date_time=(ctime.tm_year, ctime.tm_mon, ctime.tm_mday, ctime.tm_hour, ctime.tm_min, ctime.tm_sec))
+            zip_info = zipfile.ZipInfo(
+                filename=mf.file.name,
+                date_time=(ctime.tm_year, ctime.tm_mon, ctime.tm_mday, ctime.tm_hour, ctime.tm_min, ctime.tm_sec))
             zip_info.comment = info
             zip_file.writestr(zip_info, file_data.read())
 
