@@ -4,9 +4,12 @@
 
 from __future__ import absolute_import
 
+import warnings
+
 from django.conf import settings as django_settings
 from django.core.exceptions import PermissionDenied
 from django.contrib.contenttypes.models import ContentType
+from django.contrib.staticfiles.templatetags.staticfiles import static
 from django.contrib import admin
 from django.core.urlresolvers import reverse
 from django.http import HttpResponseRedirect
@@ -14,11 +17,13 @@ from django.utils.functional import curry
 from django.utils.translation import ugettext_lazy as _
 
 from feincms import ensure_completely_loaded
+from feincms import settings
 from feincms.admin import item_editor, tree_editor
 
 # ------------------------------------------------------------------------
 from .forms import PageAdminForm
 
+
 # ------------------------------------------------------------------------
 class PageAdmin(item_editor.ItemEditor, tree_editor.TreeEditor):
     class Media:
@@ -27,9 +32,6 @@ class PageAdmin(item_editor.ItemEditor, tree_editor.TreeEditor):
 
     form = PageAdminForm
 
-    # the fieldsets config here is used for the add_view, it has no effect
-    # for the change_view which is completely customized anyway
-    unknown_fields = ['template_key', 'parent', 'override_url', 'redirect_to']
     fieldset_insertion_index = 2
     fieldsets = [
         (None, {
@@ -39,8 +41,8 @@ class PageAdmin(item_editor.ItemEditor, tree_editor.TreeEditor):
                 ],
         }),
         (_('Other options'), {
-            'classes': ['collapse',],
-            'fields': unknown_fields,
+            'classes': ['collapse'],
+            'fields': ['template_key', 'parent', 'override_url', 'redirect_to'],
         }),
         # <-- insertion point, extensions appear here, see insertion_index above
         item_editor.FEINCMS_CONTENT_FIELDSET,
@@ -49,7 +51,7 @@ class PageAdmin(item_editor.ItemEditor, tree_editor.TreeEditor):
     list_display = ['short_title', 'is_visible_admin', 'in_navigation_toggle', 'template']
     list_filter = ['active', 'in_navigation', 'template_key', 'parent']
     search_fields = ['title', 'slug']
-    prepopulated_fields = { 'slug': ('title',), }
+    prepopulated_fields = {'slug': ('title',)}
 
     raw_id_fields = ['parent']
     radio_fields = {'template_key': admin.HORIZONTAL}
@@ -81,35 +83,70 @@ class PageAdmin(item_editor.ItemEditor, tree_editor.TreeEditor):
         present_fields = flatten_fieldsets(self.fieldsets)
 
         for f in self.model._meta.fields:
-            if not f.name.startswith('_') and not f.name in ('id', 'lft', 'rght', 'tree_id', 'level') and \
-                    not f.auto_created and not f.name in present_fields and f.editable:
-                self.unknown_fields.append(f.name)
+            if (not f.name.startswith('_')
+                    and not f.name in ('id', 'lft', 'rght', 'tree_id', 'level')
+                    and not f.auto_created
+                    and not f.name in present_fields
+                    and f.editable):
+                self.fieldsets[1][1]['fields'].append(f.name)
+                warnings.warn(
+                    'Automatically adding %r to %r.fieldsets. This behavior'
+                    ' is deprecated. Use add_extension_options yourself if'
+                    ' you want fields to appear in the page'
+                    ' administration.' % (f.name, self.__class__),
+                    DeprecationWarning)
             if not f.editable:
                 self.readonly_fields.append(f.name)
 
     in_navigation_toggle = tree_editor.ajax_editable_boolean('in_navigation', _('in navigation'))
 
+    def get_readonly_fields(self, request, obj=None):
+        readonly = super(PageAdmin, self).get_readonly_fields(request, obj=obj)
+        if not settings.FEINCMS_SINGLETON_TEMPLATE_CHANGE_ALLOWED:
+            if obj and obj.template and obj.template.singleton:
+                return tuple(readonly) + ('template_key',)
+        return readonly
+
     def get_form(self, *args, **kwargs):
         form = super(PageAdmin, self).get_form(*args, **kwargs)
         return curry(form, modeladmin=self)
 
     def _actions_column(self, page):
-        editable = getattr(page, 'feincms_editable', True)
+        addable = getattr(page, 'feincms_addable', True)
 
         preview_url = "../../r/%s/%s/" % (
                 ContentType.objects.get_for_model(self.model).id,
                 page.id)
         actions = super(PageAdmin, self)._actions_column(page)
-        if editable:
-            actions.insert(0, u'<a href="add/?parent=%s" title="%s"><img src="%sfeincms/img/icon_addlink.gif" alt="%s"></a>' % (
-                page.pk, _('Add child page'), django_settings.STATIC_URL, _('Add child page')))
-        actions.insert(0, u'<a href="%s" title="%s"><img src="%sfeincms/img/selector-search.gif" alt="%s" /></a>' % (
-            preview_url, _('View on site'), django_settings.STATIC_URL, _('View on site')))
 
+        if addable:
+            if not page.template.enforce_leaf:
+                actions.insert(
+                    0,
+                    u'<a href="add/?parent=%s" title="%s">'
+                    u'<img src="%s" alt="%s" />'
+                    u'</a>' % (
+                        page.pk,
+                        _('Add child page'),
+                        static('feincms/img/icon_addlink.gif'),
+                        _('Add child page'),
+                    )
+                )
+        actions.insert(
+            0,
+            u'<a href="%s" title="%s">'
+            u'<img src="%s" alt="%s" />'
+            u'</a>' % (
+                preview_url,
+                _('View on site'),
+                static('feincms/img/selector-search.gif'),
+                _('View on site'),
+            )
+        )
         return actions
 
     def add_view(self, request, **kwargs):
-        kwargs['form_url'] = request.get_full_path() # Preserve GET parameters
+        kwargs['form_url'] = request.get_full_path()  # Preserve GET parameters
         if 'translation_of' in request.GET and 'language' in request.GET:
             try:
                 original = self.model._tree_manager.get(
@@ -122,12 +159,10 @@ class PageAdmin(item_editor.ItemEditor, tree_editor.TreeEditor):
                     django_settings.LANGUAGES).get(language_code, '')
                 kwargs['extra_context'] = {
                     'adding_translation': True,
-                    'title': _(
-                        u'Add %(language)s Translation of "%(page)s"' % {
-                            'language': language,
-                            'page': original,
-                        }
-                    ),
+                    'title': _('Add %(language)s translation of "%(page)s"') % {
+                        'language': language,
+                        'page': original,
+                        },
                     'language_name': language,
                     'translation_of': original,
                 }
@@ -170,13 +205,20 @@ class PageAdmin(item_editor.ItemEditor, tree_editor.TreeEditor):
             messages.add_message(request, messages.ERROR, _("You don't have the necessary permissions to edit this object"))
         return HttpResponseRedirect(reverse('admin:page_page_changelist'))
 
+    def has_delete_permission(self, request, obj=None):
+        if not settings.FEINCMS_SINGLETON_TEMPLATE_DELETION_ALLOWED:
+            if obj and obj.template.singleton:
+                return False
+        return super(PageAdmin, self).has_delete_permission(request, obj=obj)
+
     def is_visible_admin(self, page):
         """
         Instead of just showing an on/off boolean, also indicate whether this
         page is not visible because of publishing dates or inherited status.
         """
         if not hasattr(self, "_visible_pages"):
-            self._visible_pages = list() # Sanity check in case this is not already defined
+            # Sanity check in case this is not already defined
+            self._visible_pages = list()
 
         if page.parent_id and not page.parent_id in self._visible_pages:
             # parent page's invisibility is inherited
